package com.tzld.piaoquan.wecom.job;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.odps.data.Record;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.wecom.dao.mapper.MessageAttachmentMapper;
import com.tzld.piaoquan.wecom.dao.mapper.UserMapper;
import com.tzld.piaoquan.wecom.model.bo.PushMessage;
import com.tzld.piaoquan.wecom.model.po.MessageAttachment;
import com.tzld.piaoquan.wecom.model.po.MessageAttachmentExample;
import com.tzld.piaoquan.wecom.model.po.User;
import com.tzld.piaoquan.wecom.model.po.UserExample;
import com.tzld.piaoquan.wecom.service.MessageService;
import com.tzld.piaoquan.wecom.utils.DateUtil;
import com.tzld.piaoquan.wecom.utils.MessageUtil;
import com.tzld.piaoquan.wecom.utils.OdpsUtil;
import com.tzld.piaoquan.wecom.utils.page.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

import static com.tzld.piaoquan.wecom.common.constant.RedisConstant.GUARANTEED_MINIPROGRAM_KEY;
import static com.tzld.piaoquan.wecom.common.constant.TimeConstant.MILLISECOND_DAY;

@Component
public class WeComMessageDataJob {

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private MessageAttachmentMapper messageAttachmentMapper;

    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private MessageService messageService;

    private static final int MAX_VIDEO_NUM = 3;

    //历史优质视频可推送用户列表
    List<PushMessage> goodHistoryPushList = new ArrayList<>();

    //保底视频列表
    List<Long> guaranteedVideoIdList = new ArrayList<>();

    //从缓存中获取的保底视频数量
    int getGuaranteedVideoIdNum = 0;

    //初始化操作
    void init() {
        //历史优质视频获取
        String sql = String.format("SELECT * FROM loghubods.history_good_video_can_push_user_list where dt = %s;", DateUtil.getBeforeDayDateString());
        List<Record> recordList = OdpsUtil.getOdpsData(sql);
        if (CollectionUtils.isEmpty(recordList)) {
            return;
        }
        List<PushMessage> list = new ArrayList<>();
        for (Record record : recordList) {
            PushMessage pushMessage = new PushMessage();
            Long videoId = Long.parseLong((String) record.get(0));
            Set<Long> userIds = new HashSet<>(JSONObject.parseArray((String) record.get(1), Long.class));
            pushMessage.setVideoId(videoId);
            pushMessage.setUserIds(userIds);
            list.add(pushMessage);
        }
        goodHistoryPushList = list;

        getGuaranteedVideoIdNum = 0;
        //保底视频获取
        List<Long> videoIdList = Objects.requireNonNull(redisTemplate.opsForList().range(GUARANTEED_MINIPROGRAM_KEY, 0, -1))
                .stream().map(o -> (Integer) o).map(String::valueOf).map(Long::parseLong).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(videoIdList)) {
            throw new RuntimeException("保底数据为空");
        }
        List<Long> saveVideoIds = new ArrayList<>();
        for (Long videoId : videoIdList) {
            getGuaranteedVideoIdNum++;
            MessageAttachmentExample example = new MessageAttachmentExample();
            example.createCriteria().andMiniprogramVideoIdEqualTo(videoId);
            List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(messageAttachmentList)) {
                continue;
            }
            MessageAttachment messageAttachment = messageAttachmentList.get(0);
            if (messageAttachment.getSendTime() != null
                    && DateUtil.dateDifference(new Date(), messageAttachment.getSendTime()) < 180 * MILLISECOND_DAY) {
                continue;
            }
            saveVideoIds.add(videoId);
            if (saveVideoIds.size() >= MAX_VIDEO_NUM) {
                break;
            }
        }
        if (saveVideoIds.size() < MAX_VIDEO_NUM) {
            throw new RuntimeException("保底数据不足");
        }
        guaranteedVideoIdList = saveVideoIds;
    }

    public void assemble() {
        init();
        Map<String, List<String>> res = new HashMap<>();
        UserExample example = new UserExample();
        long count = userMapper.countByExample(example);
        int page = 1;
        int pageSize = 1000;
        long totalPageSize = count / pageSize + 1;
        for (; page <= totalPageSize; page++) {
            example.setPage(new Page<>(page, pageSize));
            List<User> userList = userMapper.selectByExample(example);
            if (CollectionUtils.isEmpty(userList)) {
                continue;
            }
            for (User user : userList) {
                String pushKey = getPushKey(user);
                if (res.containsKey(pushKey)) {
                    res.get(pushKey).add(user.getExternalUserId3rdParty());
                } else {
                    res.put(pushKey, Lists.newArrayList(user.getExternalUserId3rdParty()));
                }
            }
        }
        pushMessage(res);
        saveGuaranteedVideoIdList(guaranteedVideoIdList);
    }

    public void saveGuaranteedVideoIdList(List<Long> videoIdList) {
        MessageAttachmentExample example = new MessageAttachmentExample();
        example.createCriteria().andMiniprogramVideoIdIn(videoIdList);
        List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
        for (MessageAttachment messageAttachment : messageAttachmentList) {
            MessageAttachment updateMessageAttachment = new MessageAttachment();
            updateMessageAttachment.setId(messageAttachment.getId());
            updateMessageAttachment.setSendTime(new Date());
            messageAttachmentMapper.updateByPrimaryKeySelective(updateMessageAttachment);
        }
        //移除从redis中获取的保底数据
        for (int i = 0; i < getGuaranteedVideoIdNum; i++){
            redisTemplate.opsForList().leftPop(GUARANTEED_MINIPROGRAM_KEY);
        }
    }


    public String getPushKey(User user) {
        int n = 0;
        StringBuilder key = new StringBuilder();
        for (PushMessage pushMessage : goodHistoryPushList) {
            if (pushMessage.getUserIds().contains(user.getId())) {
                if (key.length() == 0) {
                    key.append(pushMessage.getVideoId());
                    n++;
                } else {
                    key.append("_");
                    key.append(pushMessage.getVideoId());
                }
                n++;
                if (n > MAX_VIDEO_NUM) {
                    return key.toString();
                }
            }
        }
        for (Long videoId : guaranteedVideoIdList) {
            if (key.length() == 0) {
                key.append(videoId);
                n++;
            } else {
                key.append("_");
                key.append(videoId);
            }
            n++;
            if (n > MAX_VIDEO_NUM) {
                return key.toString();
            }
        }
        return key.toString();
    }

    public void pushMessage(Map<String, List<String>> map) {
        List<JSONObject> pushList = new ArrayList<>();
        //发送人员  暂时写死 只有两个
        List<String> staffExtIdList = new ArrayList<>();
        staffExtIdList.add("wogizUDQAAk5Z_5nLRD-XTAPU9poaRCw");
        staffExtIdList.add("wogizUDQAAzM5HbqIyA4k3KR6D8vKp3w");
        String text = messageService.getMessageText();
        String name = MessageUtil.getName();
        for (Map.Entry<String, List<String>> entry : map.entrySet()) {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("name", name);
            jsonObject.put("text", text);
            JSONArray attachments = new JSONArray();
            List<Long> videoIdList = Arrays.stream(entry.getKey().split("_")).map(Long::parseLong).collect(Collectors.toList());
            for (Long videoId : videoIdList) {
                JSONObject attachment = new JSONObject();
                attachment.put("msgtype", "miniprogram");
                MessageAttachmentExample example = new MessageAttachmentExample();
                example.createCriteria().andMiniprogramVideoIdEqualTo(videoId);
                List<MessageAttachment> messageAttachmentList = messageAttachmentMapper.selectByExample(example);
                if (CollectionUtils.isEmpty(messageAttachmentList)) {
                    throw new RuntimeException("附件信息查询异常");
                }
                MessageAttachment messageAttachment = messageAttachmentList.get(0);
                JSONObject miniprogram = new JSONObject();
                miniprogram.put("appid", messageAttachment.getAppid());
                miniprogram.put("title", messageAttachment.getTitle());
                miniprogram.put("page", messageAttachment.getPage());
                miniprogram.put("cover", messageAttachment.getCover());
                attachment.put("miniprogram", miniprogram);
                attachments.add(attachment);
            }
            jsonObject.put("attachments", attachments);
            List<List<String>> lists = Lists.partition(entry.getValue(), 10000);
            for (List<String> list : lists) {
                List<JSONObject> staffEuList = new ArrayList<>();
                JSONObject newJSONObject = new JSONObject();
                newJSONObject.putAll(jsonObject);
                for (String staffExtId : staffExtIdList) {
                    JSONObject staff_eu = new JSONObject();
                    staff_eu.put("staff_ext_id", staffExtId);
                    staff_eu.put("eu_ext_ids", list);
                    staffEuList.add(staff_eu);
                }
                newJSONObject.put("staff_eu_list", staffEuList);
                pushList.add(newJSONObject);
            }
        }
        if (CollectionUtils.isEmpty(pushList)) {
            throw new RuntimeException("推送视频生成失败");
        }
        for (JSONObject jsonObject : pushList) {
            messageService.pushMessage(jsonObject);
        }


    }
}
